/*********************************************************************
 *
 *                 DHT22 Communication Protocol
 *
 *********************************************************************
 * FileName:        DHT22.c
 ********************************************************************/

/****** I N C L U D E S **********************************************************/
#include "DHT22.h"
#include <plib/delays.h>
#include <pic18.h>
#include <stdio.h>

#define _XTAL_FREQ 4000000

//Varialbes para el clculo de la temperatura
char temp_lsb,temp_msb;
char hum_lsb,hum_msb;
char chks;
unsigned char Data_DHT22[5];
int k;
int temp;
int hum;
unsigned char checksum;
float Temperatura;
float Humedad;
extern char TxdataDHT22[15];
extern char TxdataDHT22_USART[50];
extern unsigned char DHT_22_flag;
/**********************************************************************
 * Function:        void drive_OW_low (void)
 * PreCondition:    None
 * Input:           None	
 * Output:          None	
 * Overview:		Configure the OW_PIN as Output and drive the OW_PIN LOW.	
 ***********************************************************************/
void drive_OW_low_dht22(void) {
    DHT_IO_PIN = 0;
    DHT_WRITE_PIN = 0; //Direccin del puerto de salida (TRIS = 0)
}

/**********************************************************************
 * Function:        void drive_OW_high (void)
 * PreCondition:    None
 * Input:           None	
 * Output:          None	
 * Overview:		Configure the OW_PIN as Output and drive the OW_PIN HIGH.	
 ***********************************************************************/
void drive_OW_high_dht22(void) {
    DHT_IO_PIN = 0; //Salida en alto
    DHT_WRITE_PIN = 1; //Direccin del puerto de salida (TRIS = 0)
}

/**********************************************************************
 * Function:        unsigned char read_OW (void)
 * PreCondition:    None
 * Input:           None	
 * Output:          Return the status of OW pin.	
 * Overview:		Configure as Input pin and Read the status of OW_PIN 	
 ***********************************************************************/
unsigned char read_OW_dht22(void) {
    unsigned char read_data = 0;

    DHT_IO_PIN = 1; //Configuro pin como entrada

    if (1 == DHT_READ_PIN) //Si PORT est en alto, sensor envia un 1
        read_data = 1;
    else //Si PORT est en bajo, sensor envia un 0
        read_data = 0;

    return read_data;
}

unsigned char DHT_22_response_start(void) {
    unsigned char read_data = 0;

    DHT_IO_PIN = 1; //Configuro pin como entrada (TRISx)

    if (1 == DHT_READ_PIN) //Si PORTx est en alto, sensor envia un 1
        read_data = 1;
    else //Si PORTx est en bajo, sensor envia un 0
        read_data = 0;

    return read_data;
}
/**********************************************************************
 * Function:        unsigned char DHT_22_Start(void)
 * PreCondition:    None
 * Input:           None	
 * Output:          Return the Presense Pulse from the slave.	
 * Overview:		Initialization sequence start with reset pulse.
 *                  This code generates reset sequence as per the protocol
 ***********************************************************************/
unsigned char DHT_22_Start(void) {
    unsigned char presence_detect;

    //drive_OW_high ();  				// Release the bus

    drive_OW_low_dht22(); // Drive the bus low

    __delay_ms(18); //Espera 18ms

    drive_OW_high_dht22(); // Release the bus

    __delay_us(22); //Espera min 20us max 40us

    presence_detect = DHT_22_response_start(); //Sample for presence pulse from slave

    if (presence_detect == 0) {

        __delay_us(80); //Espera 80us 

        presence_detect = DHT_22_response_start(); //Sample for presence pulse from slave

        __delay_us(80); //Espera 80us 

        return presence_detect;

    } else

        return 0; //Si no hay respuesta retorna 0
}

/**********************************************************************
 * Function:        unsigned char Read_byte_DHT_22 (void)
 * PreCondition:    None
 * Input:           None
 * Output:          Return the read byte from slave device
 * Overview:		This function used to read a complete byte from the slave device.
 *				   
 ***********************************************************************/

unsigned char Read_byte_DHT_22(void) {

    unsigned char loop, result = 0;
    DHT_IO_PIN = 1; //Configuro pin como entrada (TRISx)
    for (loop = 0; loop < 8; loop++) {
        while (!DHT_READ_PIN);
        __delay_us(40);
        if (DHT_READ_PIN) result |= 1 << (7 - loop);
        while (DHT_READ_PIN);
    }
    return result;
   
}

//Funciones para sensor DHT22


void Setup_port_DHT22(void) {
    DHT_WRITE_PIN = 0;
    DHT_IO_PIN = 1;
    __delay_us(500);
}

void Off_port_DHT22(void) {
    DHT_WRITE_PIN = 0;
    DHT_IO_PIN = 0;
}

void Read_Dev_DHT22(void) {
    
    //Comunicacin con sensor
    if (!DHT_22_Start()) { // Deteccin dispositivo
        //Activamos el flag de dispositivo no presente para mostrar por pantalla y reiniciar el proceso
        DHT_22_flag = 1;
    }
    //Obtenemos los datos del sensor DHT22 si el dispositivo est prsente
    if (DHT_22_flag == 0) {
        for (k = 0; k < 5; k++) {
            Data_DHT22[k] = Read_byte_DHT_22();
        }
        //Los datos recibidos del sensor DHT22
        hum_msb = Data_DHT22[0]; //Parte entera Humedad relativa
        hum_lsb = Data_DHT22[1]; //Parte decimal Humedad relativa
        temp_msb = Data_DHT22[2] & 0x7F; //Parte entera Temperatura. El msb es el signo
        temp_lsb = Data_DHT22[3]; //Parte decimal temperatura
        chks = Data_DHT22[4]; //Checksum del sensor para comprobar los datos

        //Comprobamos los datos enviados por el sensor    

        checksum = Data_DHT22[0] + Data_DHT22[1] + Data_DHT22[2] + Data_DHT22[3];

        if (checksum != chks) {
            sprintf(TxdataDHT22, "Error CKS");
            sprintf(TxdataDHT22_USART, "ERROR CHECKSUM DHT22\r\n");
        } else {
            hum = ((hum_msb) << 8) + hum_lsb;
            temp = ((temp_msb) << 8) + temp_lsb;

            Humedad = ((float) hum) / 10;
            Temperatura = ((float) temp) / 10;

            if (Data_DHT22[2] & 0x80) {
                Temperatura *= -1;
            }
            //Mensaje para enviar por LCD
            sprintf(TxdataDHT22, "%.1f %%;  %.1f% cC;    ", Humedad, Temperatura,223);
            //Mensaje para enviar por USART
            sprintf(TxdataDHT22_USART, "Humedad Relativa: %.1f%%; Temperatura: %.1f%cC;\r\n", Humedad, Temperatura,248);

        }

    } else {
    //Mensaje error de dispositivo DHT22
        
    //Mensaje para enviar por LCD
    sprintf(TxdataDHT22," DHT22  NO  DISP");
    //Mensaje para enviar por USART
    sprintf(TxdataDHT22_USART, "Dispositivo DHT22 no encontrado\r\n");    
    //Borramos flag de error de dispositivo para iniciar la deteccin de nuevo
    DHT_22_flag = 0;
    
    }
}


/********************************************************************************************
                  E N D     O F     DHT22. C  
 *********************************************************************************************/