/********************************************************************
*
*
*
*********************************************************************
* FileName:         Delay.h
* Dependencies:     See INCLUDES section below
* Processor:        PIC16LF1947
* Compiler:         HTC
* Company:          Microchip Technology, Inc.
*
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro® Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
* File Description: This file contains the function prototypes for the
*                   delay features. This file also contains defines
*                   applicable to the delay feature.
*
* Change History:
* Author               Date        Version
* Chris Tucker        5/2/11         1.1  (Initial Release)
********************************************************************/

/**
*   @file Delay.h
*   @brief
*   @defgroup
*
*/

#ifndef __DELAY_H
#define __DELAY_H


//******************************************************/
//		DELAY DEFINES
//******************************************************/
// Number Assignment, Misc.
// 1 sec = (FOSC/4) ticks

#define PLL_1SEC        10668   // [250*3]*10668 = 8,000,500; 8,000,000 = 1 Sec
#define HF_1SEC         1334    // [250*3]*1334 = 1,000,500; 1,000,000 = 1 Sec
#define MF_1SEC         167     // [250*3]*167 = 125,250; 125,000 = 1 Sec
#define LF_1SEC         11      // [250*3]*11 = 8,250; 8,000 = 1 Sec
#define XT_10MHZ_1SEC   3334    // [250*3]*3334 = 2,500,500; 2,500,000 = 1 Sec

#define PLL_1mSEC       2668    // 2668*3 = 8,004; 8,000 = 1mS
#define HF_1mSEC        334     // 334*3 = 1,002; 1,000 = 1mS
#define MF_1mSEC        42      // 42*3 = 126; 125 = 1mS
#define LF_1mSEC        3       // 3*3 = 9; 8 = 1mS
#define XT_10MHZ_1mSEC  834     // 834*3 = 2,502; 2,500 = 1ms

#define PLL_1uSEC       16      // 16 = 1us
#define HF_1uSEC        1       // 1 = 1us
#define XT_10MHZ_1uSEC  2       // 2 = 800ns
#define SCALED_1uSEC    1       // 1 = 8us [500kHz], 1 = 125us [32kHz]

//******************************************************/
//		DELAY DEFINE MACROS
//******************************************************/
// Functional Macros
#define MF_SCALE_1uSEC()        \
    do                          \
    {                           \
        if (Freq == MF)         \
        {   numSec = (numSec/8);\
        }                       \
    }   while(0);

#define LF_SCALE_1uSEC()            \
    do                              \
    {                               \
        if (Freq == LF)             \
        {   numSec = (numSec/125);  \
        }                           \
    }   while(0);

#define DELAY3_250()                \
    do                              \
    {                               \
        _delay(250);                \
        _delay(250);                \
        _delay(250);                \
    }   while(0);

#define DELAY3_1()                  \
    do                              \
    {                               \
        _delay(1);                  \
        _delay(1);                  \
        _delay(1);                  \
    }   while(0);

//******************************************************/
//		Delay Prototypes
//******************************************************/

void Delay_uSec(unsigned int);
void Delay_mSec(unsigned char);
void Delay_Sec(unsigned char);

#endif  // XLP.DELAY.h
